<?php
/**
 * User: nester_a
 * Date: 29.10.13
 */

namespace LMS\Tools\Conference\Repositories;

use LMS\Tools\Conference\Table;
use LMS\Libraries\Base;

class Booking extends Base\Repository
{

    public function __construct($options)
    {
        parent::__construct($options);
		$this->table = new Table\Booking();
    }

	/**
	 * Return count of booked conferences filtered by student id and teacher id
	 *
	 * @param int $courseID
	 * @param int $teacherID
	 * @param int $studentID
	 * @return int
	 */
	public function count($courseID, $teacherID=false, $studentID=false, $filterOptions=array())
	{
		$query = $this->db->getQuery(true);

		$tableAlias = $this->table->getTableAlias();

		$query->from($this->table);
		$query->select('COUNT(distinct '.$tableAlias.'.p_id)');
		$query->where($tableAlias.'.teacher.id <> 0');
		$query->where($tableAlias.'.course_id = '.$courseID);

		if($studentID){
			$query->where($tableAlias.'.conference_user.user_id = '.$studentID.' OR '.$tableAlias.'.public = 1');
		}

		if($teacherID){
			$query->where($tableAlias.'.teacher_id = '.$teacherID);
		}

		if(isset($filterOptions['history']) && !$filterOptions['history']){
			$query->where($tableAlias.'.to_time >= UNIX_TIMESTAMP(NOW())');
		}

		if(isset($filterOptions['future']) && !$filterOptions['future']){
			$query->where($tableAlias.'.from_time < UNIX_TIMESTAMP(NOW())');
		}

		$this->db->setQuery($query);
		return $this->db->LoadResult();
	}


	/**
	 * Returns all available booked conferences for period excluding selected $bookedConferenceId
	 *
	 * @param int $courseID
	 * @param int $teacherID
	 * @param int $bookedConferenceID
	 * @param string $fromTime
	 * @param string $toTime
	 * @return mixed
	 */
	public function isAlreadyExistsForPeriod($courseID, $teacherID, $bookedConferenceID, $fromTime, $toTime)
	{
		$query = $this->db->getQuery(true);

		$tableAlias = $this->table->getTableAlias();

		$query->from($this->table);
		$query->select('COUNT('.$tableAlias.'.p_id)');
		$query->where($tableAlias.'.teacher_id = '.$teacherID);
		$query->where($tableAlias.'.p_id <> '.$bookedConferenceID);
		$query->where('(('.$tableAlias.'.from_time > '.$fromTime.' AND '.$tableAlias.'.from_time < '.$toTime.')
                                OR
                                    ('.$tableAlias.'.to_time > '.$fromTime.' AND '.$tableAlias.'.to_time < '.$toTime.')
                                OR
                                    ('.$tableAlias.'.from_time < '.$fromTime.' AND '.$tableAlias.'.to_time > '.$fromTime.')
                                OR
                                    ('.$tableAlias.'.from_time < '.$toTime.' AND '.$tableAlias.'.to_time > '.$toTime.')
                                OR
                                    ('.$tableAlias.'.from_time = '.$fromTime.' AND '.$tableAlias.'.to_time = '.$toTime.'))');

		if($teacherID == 0){
			$query->where(''.$tableAlias.'.course_id='.$courseID);
		}

		$this->db->setQuery($query);
		return ($this->db->loadResult() > 0);
	}



	/**
	 * Function to get list of available conferences at current time
	 *
	 * @return mixed
	 */
	public function getAvailable($courseID, $timeOffset, $withTeacherNames=false)
	{
		$query = $this->db->getQuery(true);
		$tableAlias = $this->table->getTableAlias();
		$query->from($this->table);

		$query->select($tableAlias.'.*');
		if($withTeacherNames){
			$query->select($tableAlias.'.teacher.username as teacher_name');
		}

		$currentTimestamp = ( time() - $timeOffset * 60*60);

		$query->where(array(
			$tableAlias.'.course_id = ' . $this->db->quote($courseID),
			$tableAlias.'.from_time <= ' . $currentTimestamp,
			$tableAlias.'.to_time >= ' . $currentTimestamp
		));

		$this->db->setQuery( $query );

		return $this->db->loadObjectList();
	}

	/**
	 * @param $courseID
	 * @param $studentID
	 * @return mixed
	 */
	public function getStudentSoonest($courseID, $studentID)
	{
		return $this->getByFilter(array(
			'courseID' => $courseID,
			'filterStudent' => $studentID,
			'future' => true,
			'limit' => 5,
			'published' => 1
		));
	}

    /**
     * @param $courseID
     * @param $teacherID
     * @return mixed
     */
    public function getTeacherSoonest($courseID, $teacherID)
    {
        return $this->getByFilter(array(
			'courseID' => $courseID,
			'filterTeacher' => $teacherID,
			'future' => true,
			'limit' => 5,
			'published' => 1
		));
    }




	protected function beforeSave(&$data)
	{
		$this->prepareData($data);
	}

	protected function beforeSaveMultiple(&$data)
	{
		$this->prepareMultipleData($data);
	}

	protected function beforeUpdate(&$data, $conditions)
	{
		$this->prepareData($data);
	}



	protected function prepareData(&$data)
	{
		$data['course_id'] = $data['courseID'];
		$data['public'] = $data['c_public'];
	}

	protected function prepareFilter($query, &$filterOptions)
	{
		$tableAlias = $this->table->getTableAlias();
		$query->select($tableAlias.'.*, '.$tableAlias.'.teacher.id as user_teacher_id, '.$tableAlias.'.teacher.username');

		$conditions = array();
		$conditions[] = ''.$tableAlias.'.course_id = '.$filterOptions['courseID'];
		if(isset($filterOptions['filterStudent']) && $filterOptions['filterStudent'] > 0){
			$conditions[] = '(' . $tableAlias.'.conference_user.user_id = '.$filterOptions['filterStudent'].' OR '.$tableAlias.'.public = 1 )';
		}

		if(isset($filterOptions['filterTeacher']) && $filterOptions['filterTeacher'] > 0){
			$conditions[] = ''.$tableAlias.'.teacher_id = '.$filterOptions['filterTeacher'];
		}

		if(isset($filterOptions['future']) && $filterOptions['future']){
			$conditions[] = ''.$tableAlias.'.from_time > UNIX_TIMESTAMP(NOW())';
		}

		if(isset($filterOptions['future']) && !$filterOptions['future']){
			$conditions[] = ''.$tableAlias.'.from_time < UNIX_TIMESTAMP(NOW())';
		}

		if(isset($filterOptions['history']) && $filterOptions['history']){
			$conditions[] = ''.$tableAlias.'.to_time < UNIX_TIMESTAMP(NOW())';
		}

		if(isset($filterOptions['history']) && !$filterOptions['history']){
			$conditions[] = ''.$tableAlias.'.to_time > UNIX_TIMESTAMP(NOW())';
		}

		if(isset($filterOptions['published'])){
			$conditions[] = ''.$tableAlias.'.published = ' . $filterOptions['published'];
		}

		if(isset($filterOptions['order']) && $filterOptions['order'] != false){
			$query->order($filterOptions['order']);
		}else{
			$query->order($tableAlias.'.from_time');
		}

		$query->where($conditions);

		$filterOptions['limit'] = isset($filterOptions['limit']) ? $filterOptions['limit'] : 0;
		$filterOptions['offset'] = isset($filterOptions['limitStart']) ? $filterOptions['limitStart'] : 0;

		return $query;
	}

	protected function prepareMultipleData(&$data)
	{
		foreach($data as &$item){
			$this->prepareData($item);
		}

		return $data;
	}
}